# 2020-08-07 - Added instructions and option to change the direction the spikes go for fields on the z axis
# 2020-08-07 - Changed the visual part of the ball to be a small armor stand, so that the model disappears less easily (it required to change the model, which was tedious)
# 2020-08-08 - Added free ball mode to use the ball outside of volleyball, option to automatically set mode and option for keeping bright the ball rack's armor stand
# 2020-08-08 - Changed the way the ball is spawned when the ball item is thrown to spawn it in front of the player instead than at their feet
# 2020-08-09 - Added instructions to change ball item if playing in survival and link to the video

# WHAT YOU NEED TO DO FOR THE SETUP:
# 1 - decide if you want to play in a field on the x axis, in one on the z axis or in no field at all (check the "player tagging" section of this function)
# 2 - decide in your world where to put your ball rack and place there a composter block (just for visuals, feel free to use any other block if it makes sense)
# 3 - locate the function "ball_rack_volleyball" in this datapack, open it and change, in the first command, the coordinates for where the armor stand is summoned
# 4 - run the function by using in chat this command: /function volleyball:ball_rack_volleyball
# 5 - you will get a sign and a ball will be spawned in front of you: place the sign on the composter block
# You can now play; remember to sneak close to the ball to get it back and click the sign to put it back in the rack

# If your field it's on the z axis, give to the players the tag "volley_z" with this command:
# /tag @a add volley_z
# In this way, a different set of 4 commands will influence the direction of spikes
# If you want to reset the change, just remove the tag with /tag @a remove volley_z
# If you want to automatically give this tag to playing players, check the "player tagging" section of this function

# Free ball mode: the spike will always go in the direction the player is facing; useful if you want to use the ball outside of volleyball
# The spikes in the 4 cardinal directions are the same as before, but the diagonal spikes are instead fully so, instead of being directed more in the axis of the field
# This makes the ball behave realistically, but it's less useful to play volleyball, because the diagonal spikes are less impactful and more difficult to land inside the field
# To enable this mode, give to the players the tag "free_ball" with this command:
# /tag @a add free_ball
# This tag has priority over the default mode for volleyball, so you will have to remove it if you want to use the ball inside a field
# If you want to automatically give this tag to playing players, check the "player tagging" section of this function

# HOW TO PLAY:
# You decide the rules of the game, so the only thing to know is how to do different shots.
# If you just hit the ball with your hand, it will go in the direction you are facing and a bit upwards, but you can also do special skills:
# jump and sneak - the ball is launched straight and high in the air (usually used to serve);
# jump - you will spike the ball straight or a bit on the right or the left if you're looking in that direction;
# sneak - you will set the ball, which will go straight in the air for less time than when you also jump.
# To get the ball from the rack, just right click it to get it as an item
# To spawn the ball, just throw the item you got and it will spawn in front of you
# If both the ball and you are on the ground, sneak close to it to get it back as an item; this works also if the ball is floating in the water and you're just close to it
# If you have the ball as an item in your inventory, right click the sign on the ball rack to put it inside

# There can be only 1 ball at any single time; if you want more than 1 ball, you will need to replicate the commands using different tags for the ball

# This datapack uses a resource pack for the ball model; if you don't use it, it will appear like an iron block
# This means that the ball item behaves like an iron block: if you use the datpack in survival, be careful to not place it
# To eliminate the problem, consider switching the iron block with another less useful item in both the datapack and the resource pack



# Player tagging - These commands give to the players the tags used to play

# These commands make sure that only the players that are playing have the right tag to influence the game
execute as @e[tag=volleyball] at @s run tag @a[distance=..10] add volley
execute unless entity @e[tag=volleyball] run tag @a remove volley
# By default, this tag is used for fields on the x axis, but you can decide to automatically give to the players the tag for fields on the z axis or the tag for the free ball mode

# For fields an the z axis, remove the # from these two commands to activate them:
#execute as @e[tag=volleyball] at @s run tag @a[distance=..10] add volley_z
#execute unless entity @e[tag=volleyball] run tag @a remove volley_z

# For the free ball mode, remove the # from these two commands to activate them:
#execute as @e[tag=volleyball] at @s run tag @a[distance=..10] add free_ball
#execute unless entity @e[tag=volleyball] run tag @a remove free_ball


# Gameplay - These commands control how the game is played by changing the Motion tag of the ball based on how it is hit by the players

# The first command keeps constantly reset the score for the sneaking objective for players that are not playing the game
scoreboard players reset @a[tag=!volley] sneak_volley
# If the player is sneaking while hitting the ball, these commands will remove the ball's momentum on the x and z axis, making it bounce straight up
execute at @e[tag=volleyball] if entity @a[limit=1,sort=nearest,distance=..2.5,scores={sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[0] set value 0.0
execute at @e[tag=volleyball] if entity @a[limit=1,sort=nearest,distance=..2.5,scores={sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[2] set value 0.0
# To make the game playable, the ball will go higher than normal when it is hit by the player
execute at @e[tag=volleyball] if entity @a[limit=1,sort=nearest,distance=..2.5,nbt={OnGround:true},scores={hit_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[1] set value 0.7
# If the players hit the ball while jumping and sneaking, the ball will bounce really high, useful before a spike
execute at @e[tag=volleyball] if entity @a[limit=1,sort=nearest,distance=..2.5,nbt={OnGround:false},scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[1] set value 1.0
# If the players hit the ball while jumping and not sneaking, the ball will receive downwards momentum, resulting in a spike
execute at @e[tag=volleyball] if entity @a[limit=1,sort=nearest,distance=..2.5,nbt={OnGround:false},scores={hit_volley=1..}] unless entity @a[limit=1,sort=nearest,distance=..2.5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[1] set value -0.7
# The next four commands use the player orientation to add particular momentum to the ball; you will need to change these commands based on your field's orientation.
# These first two command add momentum for the spike towards the opponent area, according to the player's orientation
execute at @e[tag=volleyball] if entity @a[tag=!volley_z,tag=!free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=180..0] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[0] set value 1.0
execute at @e[tag=volleyball] if entity @a[tag=!volley_z,tag=!free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=0..180] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[0] set value -1.0
# These other two commands, instead, will also add momentum to the side if the player is enough oriented towards the right or left
execute at @e[tag=volleyball] if entity @a[tag=!volley_z,tag=!free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=-60..60] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[2] set value 0.5
execute at @e[tag=volleyball] if entity @a[tag=!volley_z,tag=!free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=120..-120] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[2] set value -0.5


# 2020-08-07
# These commands will change the direction of spikes to be on the z axis if the player has the tag "volley_z"
execute at @e[tag=volleyball] if entity @a[tag=volley_z,tag=!free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=-90..90] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[2] set value 1.0
execute at @e[tag=volleyball] if entity @a[tag=volley_z,tag=!free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=90..-90] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[2] set value -1.0
execute at @e[tag=volleyball] if entity @a[tag=volley_z,tag=!free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=-150..-30] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[0] set value 0.5
execute at @e[tag=volleyball] if entity @a[tag=volley_z,tag=!free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=30..150] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[0] set value -0.5
# If the player doesn't have the tag, the default commands for the x axis will instead work


# 2020-08-08
# These commands will direct the spike in the direction the player is facing if the player has the tag "free_ball"
# The first 4 commands are for the cardinal directions:
execute at @e[tag=volleyball] if entity @a[tag=free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=-115..-65] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[0] set value 1.0
execute at @e[tag=volleyball] if entity @a[tag=free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=65..115] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[0] set value -1.0
execute at @e[tag=volleyball] if entity @a[tag=free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=-25..25] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[2] set value 1.0
execute at @e[tag=volleyball] if entity @a[tag=free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=155..-155] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[2] set value -1.0
# The next 4 pairs of commands are for the diagonal directions
# By default, these spikes are less powerful than straight ones in case you still want to use this mode to play volleyball, but you can change that
# South-East
execute at @e[tag=volleyball] if entity @a[tag=free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=-65..-25] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[0] set value 0.5
execute at @e[tag=volleyball] if entity @a[tag=free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=-65..-25] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[2] set value 0.5
# North-East
execute at @e[tag=volleyball] if entity @a[tag=free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=-155..-115] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[0] set value 0.5
execute at @e[tag=volleyball] if entity @a[tag=free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=-155..-115] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[2] set value -0.5
# South-West
execute at @e[tag=volleyball] if entity @a[tag=free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=25..65] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[0] set value -0.5
execute at @e[tag=volleyball] if entity @a[tag=free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=25..65] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[2] set value 0.5
# North-West
execute at @e[tag=volleyball] if entity @a[tag=free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=115..155] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[0] set value -0.5
execute at @e[tag=volleyball] if entity @a[tag=free_ball,limit=1,sort=nearest,distance=..5,nbt={OnGround:false},scores={hit_volley=1..},y_rotation=115..155] unless entity @a[limit=1,sort=nearest,distance=..5,scores={hit_volley=1..,sneak_volley=1..}] run data modify entity @e[tag=volleyball,limit=1] Motion[2] set value -0.5


# After all the previous command have triggered, the objective tracking the hits on the ball is resetted
scoreboard players reset @a hit_volley
# These next two commands will give the ball back to the player if they sneak close to it
# This one will do so only if both the player and the ball are on the ground
execute at @e[tag=volleyball] if entity @a[scores={sneak_volley=1..},nbt={OnGround:true},distance=..1] unless entity @e[tag=volleyball,tag=air_ball] run tag @p add stop_volley
# This one, instead, will give the ball back if it is in the water
execute at @e[tag=volleyball] if block ~ ~0.43 ~ water if entity @a[scores={sneak_volley=1..},distance=..1] run tag @p add stop_volley
# Finally, at the end of the chain, the objective tracking the sneaking by the player is resetted
scoreboard players reset @a[scores={sneak_volley=1..}] sneak_volley



# Start of the game

# This command will kill the stand inside the ball rack once the ball has been taken from it
execute unless entity @e[tag=ball_stop,nbt={ArmorItems:[{},{},{},{id:"minecraft:iron_block",Count:1b,tag:{ball:1b}}]}] run kill @e[tag=ball_stop]

# These commands will trigger once the ball item has been thrown on the ground
# These commands will spawn the physics part and the visual part of the ball, a pig and an armor stand with the item on the head
execute as @e[type=item,nbt={Item:{tag:{ball:1b}}}] at @p rotated ~ 0 positioned ^ ^ ^1 run summon pig ~ ~ ~ {Attributes:[{Name:"generic.movement_speed",Base:0.0F},{Name:"generic.follow_range",Base:0.0F}],Silent:1,ActiveEffects:[{Id:11,Amplifier:10,Duration:2147483647,ShowParticles:false},{Id:28,Amplifier:0,Duration:2147483647,ShowParticles:false},{Id:14,Amplifier:0,Duration:2147483647,ShowParticles:false}],Tags:["volleyball","ball_volley"]}
execute as @e[type=item,nbt={Item:{tag:{ball:1b}}}] at @p rotated ~ 0 positioned ^ ^ ^1 run summon armor_stand ~ ~ ~ {Small:true,Invisible:true,Marker:true,Fire:100s,ArmorItems:[{},{},{},{id:iron_block,Count:1b,tag:{CustomModelData:2}}],Tags:["ball_volley","visual_volley"]}
# After the two part of the ball have been spawned, the ball item is killed
execute as @e[type=item,nbt={Item:{tag:{ball:1b}}}] at @s run kill @s


# End of the game

# These commands will trigger after the player has taken the ball back
# The two parts of the ball are killed, the player receives back the ball item and then the tag that started this chain is removed
execute if entity @a[tag=stop_volley] run data merge entity @e[limit=1,tag=volleyball] {Health:0f,DeathTime:19s}
execute if entity @a[tag=stop_volley] run give @p iron_block{CustomModelData:2,ball:1b,"display":{"Name":"[{\"text\":\"Volleyball\",\"color\":\"white\",\"bold\":true,\"italic\":false}]","Lore":["{\"text\":\"Throw on ground to start the game\",\"color\":\"gray\",\"italic\":true}"]}}
execute if entity @a[tag=stop_volley] run kill @e[tag=ball_volley]
execute if entity @a[tag=stop_volley] run tag @a remove stop_volley


# Visuals and Sounds

# The visual part of the ball is continuously teleported to the physics part
execute at @e[tag=volleyball] positioned ~ ~-0.586 ~ run tp @e[tag=ball_volley,tag=visual_volley] ~ ~ ~
# The visual part is usually underground, so this command make sure the ball is always bright
execute as @e[tag=visual_volley] run data merge entity @s {Fire:100s}

# If the block you choose for the ball rack makes its armor stand black, remove the # from this command to keep it bright:
#execute as @e[tag=ball_stop] run data merge entity @s {Fire:100s}
# Doing so, though, will make a flame effect appear on the ball, so it's best if you use a transparent block for the ball rack

# Between all the following commands, all the /tag are only used to make sure that the /playsound and /particle commands are played only once
execute as @e[tag=volleyball,nbt={OnGround:false}] run tag @s add air_ball
execute as @e[tag=volleyball,tag=air_ball,nbt={OnGround:true}] run tag @s add ground_ball
execute as @e[tag=volleyball,nbt={OnGround:true}] run tag @s remove air_ball
execute as @e[tag=volleyball,tag=ground_ball] at @s if block ~ ~-1 ~ sand run playsound minecraft:block.sand.place master @a[distance=..10] ~ ~ ~ 1 1
execute as @e[tag=volleyball,tag=ground_ball] at @s if block ~ ~-1 ~ sand run particle block sand ~ ~ ~ 0.3 0.3 0.3 1 25 force
execute as @e[tag=volleyball,tag=ground_ball] run tag @s remove ground_ball


# Video about this datapack: https://youtu.be/n8LKIpqOr4k
# This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International License.
# You can monetize videos about this datapack; for any other exception, like modifying the datapack, you must first contact the creator.
# This data pack can be used freely; if you do intend to use this data pack for a custom map, you have to credit the creator, PmkExpert, in a visible way.
# Contact on Reddit for any question: https://www.reddit.com/user/Pmk23
# Visit Planet Minecraft for the latest updates: https://www.planetminecraft.com/member/pmk/
# Social Medias: https://twitter.com/PmkExpert https://www.youtube.com/user/Pmkexpert